@@ -22,6 +22,7 @@ int8 = ir.IntType(8)
 int16 = ir.IntType(16)
 int32 = ir.IntType(32)
 int64 = ir.IntType(64)
+hlf = ir.HalfType()
 flt = ir.FloatType()
 dbl = ir.DoubleType()
 
@@ -47,7 +48,10 @@ class TestBase(TestCase):
 
         pattern = ''.join(map(escape, pattern))
         regex = re.sub(r'\s+', r'\\s*', pattern)
-        self.assertRegexpMatches(text, regex)
+        if PY2:
+            self.assertRegexpMatches(text, regex)
+        else:
+            self.assertRegex(text, regex)
 
     def assert_ir_line(self, line, mod):
         lines = [line.strip() for line in str(mod).splitlines()]
@@ -196,9 +200,9 @@ class TestFunction(TestBase):
         self.check_descr(self.descr(powi).strip(), """\
             declare double @"llvm.powi.f64"(double %".1", i32 %".2")""")
         self.check_descr(self.descr(memset).strip(), """\
-            declare void @"llvm.memset.p0i8.i32"(i8* %".1", i8 %".2", i32 %".3", i32 %".4", i1 %".5")""")
+            declare void @"llvm.memset.p0i8.i32"(i8* %".1", i8 %".2", i32 %".3", i1 %".4")""")
         self.check_descr(self.descr(memcpy).strip(), """\
-            declare void @"llvm.memcpy.p0i8.p0i8.i32"(i8* %".1", i8* %".2", i32 %".3", i32 %".4", i1 %".5")""")
+            declare void @"llvm.memcpy.p0i8.p0i8.i32"(i8* %".1", i8* %".2", i32 %".3", i1 %".4")""")
         self.check_descr(self.descr(assume).strip(), """\
             declare void @"llvm.assume"(i1 %".1")""")
 
@@ -1138,10 +1142,13 @@ class TestBuildInstructions(TestBase):
         a, b = builder.function.args[:2]
         tp_f = ir.FunctionType(flt, (int32, int32))
         tp_g = ir.FunctionType(dbl, (int32,), var_arg=True)
+        tp_h = ir.FunctionType(hlf, (int32, int32))
         f = ir.Function(builder.function.module, tp_f, 'f')
         g = ir.Function(builder.function.module, tp_g, 'g')
+        h = ir.Function(builder.function.module, tp_h, 'h')
         builder.call(f, (a, b), 'res_f')
         builder.call(g, (b, a), 'res_g')
+        builder.call(h, (a, b), 'res_h')
         builder.call(f, (a, b), 'res_f_fast', cconv='fastcc')
         res_f_readonly = builder.call(f, (a, b), 'res_f_readonly')
         res_f_readonly.attributes.add('readonly')
@@ -1151,6 +1158,7 @@ class TestBuildInstructions(TestBase):
             my_block:
                 %"res_f" = call float @"f"(i32 %".1", i32 %".2")
                 %"res_g" = call double (i32, ...) @"g"(i32 %".2", i32 %".1")
+                %"res_h" = call half @"h"(i32 %".1", i32 %".2")
                 %"res_f_fast" = call fastcc float @"f"(i32 %".1", i32 %".2")
                 %"res_f_readonly" = call float @"f"(i32 %".1", i32 %".2") readonly
                 %"res_fast" = call fast float @"f"(i32 %".1", i32 %".2")
@@ -1364,6 +1372,75 @@ class TestBuildInstructions(TestBase):
                 ret i16 %"c"
             """)
 
+    def test_convert_to_fp16_f32(self):
+        block = self.block(name='my_block')
+        builder = ir.IRBuilder(block)
+        a = ir.Constant(flt, 5.0)
+        b = builder.convert_to_fp16(a, name='b')
+        builder.ret(b)
+        self.check_block(block, """\
+            my_block:
+                %"b" = call i16 @"llvm.convert.to.fp16.f32"(float 0x4014000000000000)
+                ret i16 %"b"
+            """)
+
+    def test_convert_to_fp16_f32_wrongtype(self):
+        block = self.block(name='my_block')
+        builder = ir.IRBuilder(block)
+        a = ir.Constant(int16, 5)
+
+        with self.assertRaises(TypeError) as raises:
+            builder.convert_to_fp16(a, name='b')
+        self.assertIn(
+            "expected a float type, got i16",
+            str(raises.exception))
+
+    def test_convert_from_fp16_f32(self):
+        block = self.block(name='my_block')
+        builder = ir.IRBuilder(block)
+        a = ir.Constant(int16, 5)
+        b = builder.convert_from_fp16(a, name='b', to=flt)
+        builder.ret(b)
+        self.check_block(block, """\
+            my_block:
+                %"b" = call float @"llvm.convert.from.fp16.f32"(i16 5)
+                ret float %"b"
+            """)
+
+    def test_convert_from_fp16_f32_notype(self):
+        block = self.block(name='my_block')
+        builder = ir.IRBuilder(block)
+        a = ir.Constant(flt, 5.5)
+
+        with self.assertRaises(TypeError) as raises:
+            builder.convert_from_fp16(a, name='b')
+        self.assertIn(
+            "expected a float return type",
+            str(raises.exception))
+
+
+    def test_convert_from_fp16_f32_wrongtype(self):
+        block = self.block(name='my_block')
+        builder = ir.IRBuilder(block)
+        a = ir.Constant(flt, 5.5)
+
+        with self.assertRaises(TypeError) as raises:
+            builder.convert_from_fp16(a, name='b', to=flt)
+        self.assertIn(
+            "expected an i16 type, got float",
+            str(raises.exception))
+
+    def test_convert_from_fp16_f32_wrongtype2(self):
+        block = self.block(name='my_block')
+        builder = ir.IRBuilder(block)
+        a = ir.Constant(flt, 5.5)
+
+        with self.assertRaises(TypeError) as raises:
+            builder.convert_from_fp16(a, name='b', to=int16)
+        self.assertIn(
+            "expected a float type, got i16",
+            str(raises.exception))
+
     def test_cttz(self):
         block = self.block(name='my_block')
         builder = ir.IRBuilder(block)
